#ifndef STM32_REGISTERS_H
#define STM32_REGISTERS_H

/*{ spi_registers.h ******************************************************
 
 Structure definitions of stm32-registers.
 
 written by Gregor Rebel 2012
 
}*/
//{ Defines/ TypeDefs ****************************************************


//} Defines
//{ Includes *************************************************************

 // Basic set of helper functions
#include "ttc_basic.h"
//#include "stm32_gpio.h"
//#include "stm32_io.h"

#include <stdlib.h>

#define USE_STDPERIPH_DRIVER
#include "stm32f10x.h"
#include "core_cm3.h"        // StdPeripheralsLibrary/CMSIS/CM3/CoreSupport/
#include "stm32f10x_conf.h"

/*{ optional
// ADC
#include "stm32f10x_adc.h"
#include "stm32f10x_dma.h"

// Timers
#include "stm32f10x_tim.h"

#include "stm32f10x_it.h"
#include "system_stm32f10x.h"
}*/

#include "bits.h"

#include <string.h>
#include "FreeRTOS.h"
#include "task.h"

//} Includes
//{ STM32 Registers ******************************************************

typedef struct { // RCC_APB1ENR_t  APB1 peripheral clock enable register
  unsigned TIM2EN       : 1;
  unsigned TIM3EN       : 1;
  unsigned TIM4EN       : 1;
  unsigned reserved7    : 1;
  unsigned TIM6EN       : 1;
  unsigned TIM7EN       : 1;
  unsigned reserved6    : 5;
  unsigned WWDGEN       : 1;
  unsigned reserved5    : 2;
  unsigned SPI2EN       : 1;
  unsigned reserved4    : 2;
  unsigned USART2EN     : 1;
  unsigned USART3EN     : 1;
  unsigned reserved3    : 2;
  unsigned I2C1EN       : 1;
  unsigned I2C2EN       : 1;
  unsigned reserved2    : 4;
  unsigned BKPEN        : 1;
  unsigned PWREN        : 1;
  unsigned DACEN        : 1;
  unsigned CECEN        : 1;
  unsigned reserved1    : 1;
} __attribute__((__packed__)) RCC_APB1ENR_t;
typedef struct { // RCC_APB2ENR_t  APB2 peripheral clock enable register
  unsigned reserved5    : 1;
  unsigned USART1_EN    : 1;
  unsigned reserved4    : 1;
  unsigned SPI1_EN      : 1;
  unsigned TIM1_EN      : 1;
  unsigned reserved3    : 1;
  unsigned ADC1_EN      : 1;
  unsigned reserved2    : 2;
  unsigned IOPE_EN      : 1;
  unsigned IOPD_EN      : 1;
  unsigned IOPC_EN      : 1;
  unsigned IOPB_EN      : 1;
  unsigned IOPA_EN      : 1;
  unsigned reserved1    : 1;
  unsigned AFIO_EN      : 1;
} __attribute__((__packed__)) RCC_APB2ENR_t;
typedef struct { // TIMx_CR1_t     timer control register 1
  unsigned  CEN         : 1;
  unsigned  UDIS        : 1;
  unsigned  URS         : 1;
  unsigned  OPM         : 1;
  unsigned  DIR         : 1;
  unsigned  CMS         : 2;
  unsigned  ARPE        : 1;
  unsigned  CKD         : 2;
  unsigned  reserved1   : 6;
} __attribute__((__packed__)) TIMx_CR1_t ;
typedef struct { // TIMx_CR2_t     timer control register 2
  unsigned  CCPC        : 1;
  unsigned  reserved2   : 1;
  unsigned  CCUS        : 1;
  unsigned  CCDS        : 1;
  unsigned  MMS         : 2;
  unsigned  TI1S        : 1;
  unsigned  OIS1        : 1;
  unsigned  OIS1N       : 1;
  unsigned  OIS2        : 1;
  unsigned  OIS2N       : 1;
  unsigned  OIS3        : 1;
  unsigned  OIS3N       : 1;
  unsigned  OIS4        : 1;
  unsigned  reserved1   : 1;
} __attribute__((__packed__)) TIMx_CR2_t;
typedef struct { // TIMx_SMCR_t    timer slave mode control register
  unsigned  SMS         : 3;
  unsigned  TS          : 3;
  unsigned  MSM         : 1;
  unsigned  ETF         : 4;
  unsigned  ETPS        : 2;
  unsigned  EXE         : 1;
  unsigned  ETP         : 1;
} __attribute__((__packed__)) TIMx_SMCR_t;
typedef struct { // TIMx_DIER_t    timer DMA/interrupt enable register
  unsigned  UIE         : 1;
  unsigned  CC1IE       : 1;
  unsigned  CC2IE       : 1;
  unsigned  CC3IE       : 1;
  unsigned  CC4IE       : 1;
  unsigned  COMIE       : 1;
  unsigned  TIE         : 1;
  unsigned  BIE         : 1;
  unsigned  UDE         : 1;
  unsigned  CC1DE       : 1;
  unsigned  CC2DE       : 1;
  unsigned  CC3DE       : 1;
  unsigned  CC4DE       : 1;
  unsigned  COMDE       : 1;
  unsigned  TDE         : 1;
  unsigned  reserved1   : 1;
} __attribute__((__packed__)) TIMx_DIER_t;
typedef struct { // TIMx_SR_t      timer status register
  unsigned  UIF         : 1;
  unsigned  CC1IF       : 1;
  unsigned  CC2IF       : 1;
  unsigned  CC3IF       : 1;
  unsigned  CC4IF       : 1;
  unsigned  COMIF       : 1;
  unsigned  TIF         : 1;
  unsigned  BIF         : 1;
  unsigned  reserved2   : 1;
  unsigned  CC1OF       : 1;
  unsigned  CC2OF       : 1;
  unsigned  CC3OF       : 1;
  unsigned  CC4OF       : 1;
  unsigned  reserved1   : 3;
} __attribute__((__packed__)) TIMx_SR_t;
typedef struct { // TIMx_EGR_t     timer event generation register
  unsigned  UG          : 1;
  unsigned  CC1G        : 1;
  unsigned  CC2G        : 1;
  unsigned  CC3G        : 1;
  unsigned  CC4G        : 1;
  unsigned  COMG        : 1;
  unsigned  TG          : 1;
  unsigned  BG          : 1;
  unsigned  reserved1   : 8;
} __attribute__((__packed__)) TIMx_EGR_t;
typedef struct { // TIMx_CCMR1_t   timer capture/compare register 1
  unsigned  CC1S        : 2;
  unsigned  OC1FE       : 1;
  unsigned  OC1PE       : 1;
  unsigned  OC1M        : 3;
  unsigned  OC1CE       : 1;
  unsigned  CC2S        : 2;
  unsigned  OC2FE       : 1;
  unsigned  OC2PE       : 1;
  unsigned  OC2M        : 3;
  unsigned  OC2CE       : 1;
} __attribute__((__packed__)) TIMx_CCMR1_t;
typedef struct { // TIMx_CCMR2_t   timer capture/compare register 2
  unsigned  CC3S        : 2;
  unsigned  OC3FE       : 1;
  unsigned  OC3PE       : 1;
  unsigned  OC3M        : 3;
  unsigned  OC3CE       : 1;
  unsigned  CC4S        : 2;
  unsigned  OC4FE       : 1;
  unsigned  OC4PE       : 1;
  unsigned  OC4M        : 3;
  unsigned  OC4CE       : 1;
} __attribute__((__packed__)) TIMx_CCMR2_t;
typedef struct { // TIMx_CCER_t    timer DMA/interrupt enable register
  unsigned  CC1E      : 1;
  unsigned  CC1P      : 1;
  unsigned  CC1NE     : 1;
  unsigned  CC1NP     : 1;
  unsigned  CC2E      : 1;
  unsigned  CC2P      : 1;
  unsigned  CC2NE     : 1;
  unsigned  CC2NP     : 1;
  unsigned  CC3E      : 1;
  unsigned  CC3P      : 1;
  unsigned  CC3NE     : 1;
  unsigned  CC3NP     : 1;
  unsigned  CC4E      : 1;
  unsigned  CC4P      : 1;
  unsigned  reserved1 : 2;
} __attribute__((__packed__)) TIMx_CCER_t;
typedef struct { // TIMx_BDTR_t    timer break and dead-time register
  unsigned  DTG         : 8;
  unsigned  LOCK        : 2;
  unsigned  OSSI        : 1;
  unsigned  OSSR        : 1;
  unsigned  BKE         : 1;
  unsigned  BKP_        : 1;
  unsigned  AOE         : 1;
  unsigned  MOE         : 1;
} __attribute__((__packed__)) TIMx_BDTR_t;
typedef struct { // TIMx_DCR_t     timer DMA control register
  unsigned  DBA         : 5;
  unsigned  reserved2   : 3;
  unsigned  DBL         : 5;
  unsigned  reserved1   : 1;
} __attribute__((__packed__)) TIMx_DCR_t;
typedef struct { // TIMER_t        timer 1 registers
  TIMx_CR1_t    CR1;
  u16_t         RESERVED0;
  TIMx_CR2_t    CR2;
  u16_t         RESERVED1;
  TIMx_SMCR_t   SMCR;
  u16_t         RESERVED2;
  TIMx_DIER_t   DIER;
  u16_t         RESERVED3;
  TIMx_SR_t     SR;
  u16_t         RESERVED4;
  TIMx_EGR_t    EGR;
  u16_t         RESERVED5;
  TIMx_CCMR1_t  CCMR1;
  u16_t         RESERVED6;
  TIMx_CCMR2_t  CCMR2;
  u16_t         RESERVED7;
  TIMx_CCER_t   CCER;
  u16_t         RESERVED8;
  u16_t         CNT;
  u16_t         RESERVED9;
  u16_t         PSC;
  u16_t         RESERVED10;
  u16_t         ARR;
  u16_t         RESERVED11;
  u16_t         RCC_; // RCC already defined elsewhere
  u16_t         RESERVED12;
  u16_t         CCR1;
  u16_t         RESERVED13;
  u16_t         CCR2;
  u16_t         RESERVED14;
  u16_t         CCR3;
  u16_t         RESERVED15;
  u16_t         CCR4;
  u16_t         RESERVED16;
  TIMx_BDTR_t   BDTR;
  u16_t         RESERVED17;
  TIMx_DCR_t    DCR;
  u16_t         RESERVED18;
  u16_t         DMAR;
  u16_t         RESERVED19;
} TIMER_t;

typedef struct { // USART_SR_t     USART Status Register
  unsigned PE   : 1; // Parity Error
  unsigned FE   : 1; // Framing Error
  unsigned NE   : 1; // Noise Error
  unsigned ORE  : 1; // Overrun Error
  unsigned IDLE : 1; // IDLE line detected
  unsigned RXNE : 1; // Read Data Register not empty
  unsigned TC   : 1; // Transmission complete
  unsigned TXE  : 1; // Transmission Data Register empty
  unsigned LBD  : 1; // Lin break detection flag
  unsigned CTS  : 1; // CTS Flag
  unsigned Reserved1: 6;
} __attribute__((__packed__)) USART_SR_t;
typedef struct { // USART_DR_t     USART Data Register
  unsigned Data: 9;
  unsigned Reserved1: 23;
} __attribute__((__packed__)) USART_DR_t;
typedef struct { // USART_BRR_t    USART Baud Rate Register
  unsigned DIV_Fraction: 4;
  unsigned DIV_Mantissa: 12;
} __attribute__((__packed__)) USART_BRR_t;
typedef struct { // USART_CR1_t    USART Control Register 1

  unsigned SBK: 1;    // Send break
  unsigned RWU: 1;    // Receiver wakeup
  unsigned RE: 1;     // Receiver enable
  unsigned TE: 1;     // Transmitter enable
  unsigned IDLEIE: 1; // IDLE interrupt enable
  unsigned RXNEIE: 1; // RXNE interrupt enable
  unsigned TCIE: 1;   // Transmission complete interrupt enable
  unsigned TXEIE: 1;  // TXE interrupt enable
  unsigned PEIE: 1;   // PE interrupt enable
  unsigned PS: 1;     // Parity selection
  unsigned PCE: 1;    // Parity control enable
  unsigned WAKE: 1;   // Wakeup method
  unsigned M: 1;      // Word length
  unsigned UE: 1;     // USART enable
  unsigned Reserved1: 18;

} __attribute__((__packed__)) USART_CR1_t;
typedef struct { // USART_CR2_t    USART Control Register 2

  unsigned ADD: 4;        // Address of the USART node
  unsigned Reserved1 : 1; // forced by hardware to 0.
  unsigned LBDL: 1;       // lin break detection length
  unsigned LBDIE: 1;      // LIN break detection interrupt enable
  unsigned Reserved2 : 1; // forced by hardware to 0.
  unsigned LBCL: 1;       // Last bit clock pulse
  unsigned CPHA: 1;       // Clock phase
  unsigned CPOL: 1;       // Clock polarity
  unsigned CLKEN: 1;      // Clock enable
  unsigned STOP: 2;       // STOP bits
  unsigned LINEN: 1;      // LIN mode enable
  unsigned Reserved3 :17; // forced by hardware to 0.

} __attribute__((__packed__)) USART_CR2_t;
typedef struct { // USART_CR3_t    USART Control Register 3

  unsigned EIE: 1;   // Error interrupt enable
  unsigned IREN: 1;  // IrDA mode enable
  unsigned IRLP: 1;  // IrDA low-power
  unsigned HDSEL: 1; // Half-duplex selection
  unsigned NACK: 1;  // Smartcard NACK enable
  unsigned SCEN: 1;  // Smartcard mode enable
  unsigned DMAR: 1;  // DMA enable receiver
  unsigned DMAT: 1;  // DMA enable transmitter
  unsigned RTSE: 1;  // RTS enable
  unsigned CTSE: 1;  // CTS enable
  unsigned CTSIE: 1; // CTS interrupt enable
  unsigned Reserved : 21; // forced by hardware to 0.

} __attribute__((__packed__)) USART_CR3_t;
typedef struct { // USART_GPTR_t   USART Guard Time and Prescaler Register
  unsigned PSC : 8;       // Prescaler
  unsigned GT  : 8;       // Guard Time
  unsigned Reserved : 16; // forced by hardware to 0.
} __attribute__((__packed__)) USART_GPTR_t;
typedef struct { // USART_t        USART registers
  USART_SR_t   USART_SR;
  USART_DR_t   USART_DR;
  USART_BRR_t  USART_BRR;
  USART_CR1_t  USART_CR1;
  USART_CR2_t  USART_CR2;
  USART_CR3_t  USART_CR3;
  USART_GPTR_t USART_GPTR;
} __attribute__((__packed__)) USART_t;

typedef struct { // SPI_CR1        SPI Control Register 1 (->RM0008 p.715)
  
  unsigned CPHA     :  1; // Bit 0 CPHA: Clock phase
  unsigned CPOL     :  1; // Bit 1 CPOL: Clock polarity
  unsigned MSTR     :  1; // Bit 2 MSTR: Master selection
  unsigned BR       :  3; // Bits 5:3 BR[2:0]: Baud rate control
  unsigned SPE      :  1; // Bit 6 SPE: SPI enable
  unsigned LSBFIRST :  1; // Bit 7 LSBFIRST: Frame format
  unsigned SSI      :  1; // Bit 8 SSI: Internal slave select
  unsigned SSM      :  1; // Bit 9 SSM: Software slave management
  unsigned RXONLY   :  1; // Bit 10 RXONLY: Receive only
  unsigned DFF      :  1; // Bit 11 DFF: Data frame format
  unsigned CRCNEXT  :  1; // Bit 12 CRCNEXT: CRC transfer next
  unsigned CRCEN    :  1; // Bit 13 CRCEN: Hardware CRC calculation enable
  unsigned BIDIOE   :  1; // Bit 14 BIDIOE: Output enable in bidirectional mode
  unsigned BIDIMODE :  1; // Bit 15 BIDIMODE: Bidirectional data mode enable
  unsigned Reserved : 16; // kept at zero


} __attribute__((__packed__)) SPI_CR1_t;
typedef struct { // SPI_CR2        SPI Control Register 2 (->RM0008 p.716)

  unsigned RXDMAEN   :  1; // Bit 0 RXDMAEN: Rx buffer DMA enable
  unsigned TXDMAEN   :  1; // Bit 1 TXDMAEN: Tx buffer DMA enable
  unsigned SSOE      :  1; // Bit 2 SSOE: SS output enable
  unsigned Reserved1 :  2; // Bits 4:3 Reserved, must be kept at reset value.
  unsigned ERRIE     :  1; // Bit 5 ERRIE: Error interrupt enable
  unsigned RXNEIE    :  1; // Bit 6 RXNEIE: RX buffer not empty interrupt enable
  unsigned TXEIE     :  1; // Bit 7 TXEIE: Tx buffer empty interrupt enable
  unsigned Reserved2 : 24; // Bits 15:8 Reserved, must be kept at reset value.

} __attribute__((__packed__)) SPI_CR2_t;
typedef struct { // SPI_SR         SPI Status Register (->RM0008 p.717)
  
  unsigned RXNE      :  1; // Bit 0 RXNE: Receive buffer not empty
  unsigned TXE       :  1; // Bit 1 TXE: Transmit buffer empty
  unsigned CHSIDE    :  1; // Bit 2 CHSIDE: Channel side
  unsigned UDR       :  1; // Bit 3 UDR: Underrun flag
  unsigned CRCERR    :  1; // Bit 4 CRCERR: CRC error flag
  unsigned MODF      :  1; // Bit 5 MODF: Mode fault
  unsigned OVR       :  1; // Bit 6 OVR: Overrun flag
  unsigned BSY       :  1; // Bit 7 BSY: Busy flag
  unsigned Reserved  : 24; // Bits 15:8 Reserved, must be kept at reset value.

} __attribute__((__packed__)) SPI_SR_t;
typedef struct { // SPI_DR         SPI Data Register (->RM0008 p.718)
  unsigned Data     : 16;        // data received or transmitted
  unsigned Reserved : 16;        // kept at zero
} __attribute__((__packed__)) SPI_DR_t;
typedef struct { // SPI_CRC        SPI CRC Register (->RM0008 p.718)
  unsigned Polynom  : 16;        // polynomial used for CRC calculation 
  unsigned Reserved : 16;        // kept at zero
} __attribute__((__packed__)) SPI_CRC_t;
typedef struct { // SPI_RX_CRC     SPI Computed CRC value of received data (->RM0008 p.719)
  unsigned Value    : 16;        // calculated CRC value (only lower 8 bits valid in 8-bit mode)
  unsigned Reserved : 16;        // kept at zero
} __attribute__((__packed__)) SPI_RX_CRC_t;
typedef struct { // SPI_TX_CRC     SPI Computed CRC value of transmitted data (->RM0008 p.719)
  unsigned Value    : 16;        // calculated CRC value (only lower 8 bits valid in 8-bit mode)
  unsigned Reserved : 16;        // kept at zero
} __attribute__((__packed__)) SPI_TX_CRC_t;
typedef struct { // SPI_I2SCFGR    SPI I2S-Configuration Register (->RM0008 p.720)
  
  unsigned CHLEN      :  1; // Bit 0 CHLEN: Channel length (number of bits per audio channel)
  unsigned DATLEN     :  2; // Bit 2:1 DATLEN: Data length to be transferred
  unsigned CKPOL      :  1; // Bit 3 CKPOL: Steady state clock polarity
  unsigned I2SSTD     :  1; // Bit 5:4 I2SSTD: I2S standard selection
  unsigned Reserved1  :  1; // Bit 6 Reserved: forced at 0 by hardware
  unsigned PCMSYNC    :  1; // Bit 7 PCMSYNC: PCM frame synchronization
  unsigned I2SCFG     :  2; // Bit 9:8 I2SCFG: I2S configuration mode
  unsigned I2SE       :  1; // Bit 10 I2SE: I2S Enable
  unsigned I2SMOD     :  1; // Bit 11 I2SMOD: I2S mode selection
  unsigned Reserved2  : 20; // Bits 15:12 Reserved, must be kept at reset value.

} __attribute__((__packed__)) SPI_I2SCFGR_t;
typedef struct { // SPI_I2SPR      SPI I2S Prescaler Register (->RM0008 p.721)

  unsigned I2SDIV    :  8; // Bit 7:0 I2SDIV: I2S Linear prescaler
  unsigned ODD       :  1; // Bit 8 ODD: Odd factor for the prescaler
  unsigned MCKOE     :  1; // Bit 9 MCKOE: Master clock output enable
  unsigned Reserved  : 22; // Bits 15:10 Reserved, must be kept at reset value.
  
} __attribute__((__packed__)) SPI_I2SPR_t;
typedef struct { // SPI_t
  SPI_CR1_t     CR1;
  SPI_CR2_t     CR2;
  SPI_SR_t      SR;
  SPI_DR_t      DR;
  SPI_CRC_t     CRC_Polynom;
  SPI_RX_CRC_t  RX_CRC;
  SPI_TX_CRC_t  TX_CRC;
  SPI_I2SCFGR_t I2SCFGR;
  SPI_I2SPR_t   I2SPR;
} __attribute__((__packed__)) SPI_t;

//} STM32 Registers

#endif //STM32_REGISTERS_H
