/*{ stm32_spi::.c ************************************************
 
                      The ToolChain
                      
   Universal Synchronous Asynchronous Receiver and Transmitters (stm32_SPIs)
   for ARM CortexM3 microcontrollers.
   Currently tested to comply to STM32F10x types.
   
   written by Gregor Rebel 2012
   
   Note: See ttc_spi.h for description of architecture independent SPI implementation.
 
}*/

#include "stm32_spi.h"

SPI_t* stm32_SPIs[TTC_AMOUNT_SPIS];
u8_t ss_Initialized=0;

//{ Function definitions *************************************************

void stm32_SPI_ResetAll() {
  memset(stm32_SPIs, 0, sizeof(SPI_TypeDef*) * TTC_AMOUNT_SPIS);
  ss_Initialized=1;
}
ttc_spi_errorcode_e stm32_getSPI_Defaults(u8_t SPI_Index, ttc_spi_generic_t* SPI_Generic) {
    if (!ss_Initialized) stm32_SPI_ResetAll();
#ifndef TTC_NO_ARGUMENT_CHECKS //{
    // assumption: *SPI_Generic has been zeroed

    Assert(SPI_Index > 0,    ec_InvalidArgument);
    Assert(SPI_Generic != NULL, ec_NULL);
    if (SPI_Index > TTC_AMOUNT_SPIS) return tue_DeviceNotFound;
#endif //}

    SPI_Generic->Flags.All                        = 0;
    SPI_Generic->Flags.Bits.Master                = 1;
    SPI_Generic->Flags.Bits.MultiMaster           = 0;
    SPI_Generic->Flags.Bits.ClockIdleHigh         = 1;
    SPI_Generic->Flags.Bits.ClockPhase2ndEdge     = 1;
    SPI_Generic->Flags.Bits.Simplex               = 0;
    SPI_Generic->Flags.Bits.Bidirectional         = 1;
    SPI_Generic->Flags.Bits.Receive               = 1;
    SPI_Generic->Flags.Bits.Transmit              = 1;
    SPI_Generic->Flags.Bits.WordSize16            = 0;
    SPI_Generic->Flags.Bits.HardwareNSS           = 1;
    SPI_Generic->Flags.Bits.CRC8                  = 0;
    SPI_Generic->Flags.Bits.CRC16                 = 0;
    SPI_Generic->Flags.Bits.TxDMA                 = 0;
    SPI_Generic->Flags.Bits.RxDMA                 = 0;
    SPI_Generic->Flags.Bits.FirstBitMSB           = 1;
    SPI_Generic->Flags.Bits.Irq_Error             = 0;
    SPI_Generic->Flags.Bits.Irq_Received          = 0;
    SPI_Generic->Flags.Bits.Irq_Transmitted       = 0;

    SPI_Generic->CRC_Polynom = 0x7;
    SPI_Generic->Layout      = 0;
    
    RCC_ClocksTypeDef RCC_Clocks;
    RCC_GetClocksFreq(&RCC_Clocks);
    
    SPI_Generic->BaudRatePrescaler   = 16;        // medium speed
    SPI_Generic->BaudRate  = RCC_Clocks.PCLK2_Frequency / SPI_Generic->BaudRatePrescaler; 

    return tue_OK;
}
ttc_spi_errorcode_e stm32_getSPI_Features(u8_t SPI_Index, ttc_spi_generic_t* SPI_Generic) {
    if (!ss_Initialized) stm32_SPI_ResetAll();
#ifndef TTC_NO_ARGUMENT_CHECKS //{
    // assumption: *SPI_Generic has been zeroed

    Assert(SPI_Index > 0,    ec_InvalidArgument);
    Assert(SPI_Generic != NULL, ec_NULL);
    if (SPI_Index > TTC_AMOUNT_SPIS) return tue_DeviceNotFound;
#endif //}

    SPI_Generic->Flags.All                        = 0;
    SPI_Generic->Flags.Bits.Master                = 1;
    SPI_Generic->Flags.Bits.MultiMaster           = 1;
    SPI_Generic->Flags.Bits.ClockIdleHigh         = 1;
    SPI_Generic->Flags.Bits.ClockPhase2ndEdge     = 1;
    SPI_Generic->Flags.Bits.Simplex               = 1;
    SPI_Generic->Flags.Bits.Bidirectional         = 1;
    SPI_Generic->Flags.Bits.Receive               = 1;
    SPI_Generic->Flags.Bits.Transmit              = 1;
    SPI_Generic->Flags.Bits.WordSize16            = 1;
    SPI_Generic->Flags.Bits.HardwareNSS           = 1; // BUG known for STM32
    SPI_Generic->Flags.Bits.CRC8                  = 1;
    SPI_Generic->Flags.Bits.CRC16                 = 1;
    SPI_Generic->Flags.Bits.FirstBitMSB           = 1;
    SPI_Generic->Flags.Bits.TxDMA                 = 0; // ToDo
    SPI_Generic->Flags.Bits.RxDMA                 = 0; // ToDo
    SPI_Generic->Flags.Bits.Irq_Error             = 0; // ToDo
    SPI_Generic->Flags.Bits.Irq_Received          = 0; // ToDo
    SPI_Generic->Flags.Bits.Irq_Transmitted       = 0; // ToDo

    SPI_Generic->CRC_Polynom = 0xffff;
    SPI_Generic->Layout      = 0;
    
    RCC_ClocksTypeDef RCC_Clocks;
    RCC_GetClocksFreq(&RCC_Clocks);
    
    SPI_Generic->BaudRatePrescaler   = 255;                             // max divider
    SPI_Generic->BaudRate  = RCC_Clocks.PCLK2_Frequency / 2;  // max frequency

    SPI_TypeDef* SPI_Base = NULL;
    switch (SPI_Index) {           // find corresponding SPI as defined by makefile.100_board_*
#ifdef TTC_SPI1
      case 1: SPI_Base = TTC_SPI1; break;
#endif
#ifdef TTC_SPI2
      case 2: SPI_Base = TTC_SPI2; break;
#endif
#ifdef TTC_SPI3
      case 3: SPI_Base = TTC_SPI3; break;
#endif
      default: Assert(0, ec_UNKNOWN); break; // No TTC_SPIn defined! Check your makefile.100_board_* file!
    }
    switch ( (u32_t) SPI_Base ) {  // find amount of available remapping-layouts (-> RM0008 p. 176)
    case (u32_t) SPI1:
        SPI_Generic->Layout=1;     // SPI1 provides 1 alternate pin layout
        break;
    case (u32_t) SPI2:
        SPI_Generic->Layout=0;     // SPI2 provides 0 alternate pin layouts
    case (u32_t) SPI3:
#ifdef STM32F10X_CL
            SPI_Generic->Layout=1; // SPI3 provides 1 alternate pin layout in Connection Line Devices
#else
            SPI_Generic->Layout=0; // SPI3 provides 0 alternate pin layout
#endif
        break;
      default: Assert(0, ec_UNKNOWN);
    }
    
    switch (SPI_Index) {           // disable features requiring unconfigured pins
      case 1: {
          #ifndef TTC_SPI1_MOSI
            SPI_Generic->Flags.Bits.Transmit = 0;
          #endif
          #ifndef TTC_SPI1_MISO
            SPI_Generic->Flags.Bits.Receive  = 0;
          #endif
          #ifndef TTC_SPI1_NSS
            SPI_Generic->Flags.Bits.HardwareNSS  = 1;
          #endif
          #ifndef TTC_SPI1_SCK
            SPI_Generic->Flags.Bits.Transmit = 0;
            SPI_Generic->Flags.Bits.Receive  = 0;
          #endif
          
          break;
      }
      case 2: {
          #ifndef TTC_SPI2_MOSI
            SPI_Generic->Flags.Bits.Transmit = 0;
          #endif
          #ifndef TTC_SPI2_MISO
            SPI_Generic->Flags.Bits.Receive  = 0;
          #endif
          #ifndef TTC_SPI2_NSS
            SPI_Generic->Flags.Bits.HardwareNSS  = 0;
          #endif
          #ifndef TTC_SPI2_SCK
            SPI_Generic->Flags.Bits.Transmit = 0;
            SPI_Generic->Flags.Bits.Receive  = 0;
          #endif
          
          break;
      }
      case 3: {
          #ifndef TTC_SPI3_MOSI
            SPI_Generic->Flags.Bits.Transmit = 0;
          #endif
          #ifndef TTC_SPI3_MISO
            SPI_Generic->Flags.Bits.Receive  = 0;
          #endif
          #ifndef TTC_SPI3_NSS
            SPI_Generic->Flags.Bits.HardwareNSS  = 0;
          #endif
          #ifndef TTC_SPI3_SCK
            SPI_Generic->Flags.Bits.Transmit = 0;
            SPI_Generic->Flags.Bits.Receive  = 0;
          #endif

          break;
      }
      default: Assert(0, ec_UNKNOWN); return tue_DeviceNotFound;; // SPI misconfigured! Check your makefile.100_board_* file!
    }
    
    return tue_OK;
}
ttc_spi_errorcode_e stm32_initSPI(u8_t SPI_Index, ttc_spi_generic_t* SPI_Generic, stm32_spi_architecture_t* SPI_Arch) {
    if (!ss_Initialized) stm32_SPI_ResetAll();
#ifndef TTC_NO_ARGUMENT_CHECKS //{
    // assumption: *SPI_Arch has been zeroed

    Assert(SPI_Generic != NULL, ec_NULL);
#endif //}

    if (1) { // validate SPI_Features
        ttc_spi_generic_t SPI_Features;
        ttc_spi_errorcode_e Error=stm32_getSPI_Features(SPI_Index, &SPI_Features);
        if (Error) return Error;
        SPI_Generic->Flags.All &= SPI_Features.Flags.All; // mask out unavailable flags
        if (SPI_Generic->Layout > SPI_Features.Layout)
            SPI_Generic->Layout = SPI_Features.Layout;
        if (SPI_Generic->BaudRatePrescaler > SPI_Features.BaudRatePrescaler)
            SPI_Generic->BaudRatePrescaler = SPI_Features.BaudRatePrescaler;
        if (SPI_Generic->BaudRate > SPI_Features.BaudRate)
            SPI_Generic->BaudRate = SPI_Features.BaudRate;
    }
    switch (SPI_Index) {                // find SPI corresponding to SPI_index as defined by makefile.100_board_*
#ifdef TTC_SPI1
      case 1: SPI_Arch->Base=TTC_SPI1; break;
#endif
#ifdef TTC_SPI2
      case 2: SPI_Arch->Base=TTC_SPI2; break;
#endif
#ifdef TTC_SPI3
      case 3: SPI_Arch->Base=TTC_SPI3; break;
#endif
      default: Assert(0, ec_UNKNOWN); break; // No TTC_SPIn defined! Check your makefile.100_board_* file!
    }
    if (1) { // determine pin layout
 //#if defined(STM32F10X_LD) || defined(STM32F10X_LD_VL) || defined(STM32F10X_MD) || defined(STM32F10X_MD_VL) || defined(STM32F10X_HD) || defined(STM32F10X_HD_VL) || defined(STM32F10X_XL) || defined(STM32F10X_CL)
        stm32_Port_t PortMOSI;
        switch (SPI_Index) { // check if board has defined the MOSI-pin of SPI to init
#ifdef TTC_SPI1_MOSI
        case 1:
            stm32_loadPort(&PortMOSI, TTC_SPI1_MOSI);
            break;
#endif
#ifdef TTC_SPI2_MOSI
        case 2:
            stm32_loadPort(&PortMOSI, TTC_SPI2_MOSI);
            break;
#endif
#ifdef TTC_SPI3_MOSI
        case 3:
            stm32_loadPort(&PortMOSI, TTC_SPI3_MOSI);
            break;
#endif
        default: Assert(0, ec_UNKNOWN); break; // No TTC_SPIn_MOSI defined! Check your makefile.100_board_* file!
        }

        switch ( (u32_t) SPI_Arch->Base ) { // determine pin layout (-> RM0008 p.176)
          case (u32_t) SPI1: { // SPI1
              if (PortMOSI.Pin == 15) SPI_Generic->Layout = 1;
              else                    SPI_Generic->Layout = 0;
  
              if (SPI_Generic->Layout) { // alternate pin remapping (-> ST RM0041 STM32F100xx advanced ARM-based 32-bit MCUs p.118)
                  SPI_Arch->PortNSS.GPIOx  = GPIOA;
                  SPI_Arch->PortNSS.Pin    = 15;
                  SPI_Arch->PortSCK.GPIOx  = GPIOB;
                  SPI_Arch->PortSCK.Pin    = 3;
                  SPI_Arch->PortMISO.GPIOx = GPIOB;
                  SPI_Arch->PortMISO.Pin   = 4;
                  SPI_Arch->PortMOSI.GPIOx = GPIOA;
                  SPI_Arch->PortMOSI.Pin   = 5;
              }
              else {                     // no pin remapping
                  SPI_Arch->PortNSS.GPIOx  = GPIOA;
                  SPI_Arch->PortNSS.Pin    = 4;
                  SPI_Arch->PortSCK.GPIOx  = GPIOA;
                  SPI_Arch->PortSCK.Pin    = 5;
                  SPI_Arch->PortMISO.GPIOx = GPIOA;
                  SPI_Arch->PortMISO.Pin   = 6;
                  SPI_Arch->PortMOSI.GPIOx = GPIOA;
                  SPI_Arch->PortMOSI.Pin   = 7;
              }
  
              break;
          }
          case (u32_t) SPI2: { // SPI2 has no alternative layout
  
              SPI_Arch->PortMOSI.GPIOx = GPIOB;
              SPI_Arch->PortMOSI.Pin   = 15;
              SPI_Arch->PortMISO.GPIOx = GPIOB;
              SPI_Arch->PortMISO.Pin   = 14;
              SPI_Arch->PortSCK.GPIOx  = GPIOB;
              SPI_Arch->PortSCK.Pin    = 13;
              SPI_Arch->PortNSS.GPIOx  = GPIOB;
              SPI_Arch->PortNSS.Pin    = 12;
              break;
          }
          case (u32_t) SPI3: { // SPI3
              if (PortMOSI.Pin == 12)      SPI_Generic->Layout=1; // pin remapping
              else                         SPI_Generic->Layout=0; // no pin remapping
  
              if (SPI_Generic->Layout) { // alternate pin remapping (-> ST RM0041 STM32F100xx advanced ARM-based 32-bit MCUs p.118)
                  SPI_Arch->PortNSS.GPIOx  = GPIOA;
                  SPI_Arch->PortNSS.Pin    = 4;
                  SPI_Arch->PortSCK.GPIOx  = GPIOC;
                  SPI_Arch->PortSCK.Pin    = 10;
                  SPI_Arch->PortMISO.GPIOx = GPIOC;
                  SPI_Arch->PortMISO.Pin   = 11;
                  SPI_Arch->PortMOSI.GPIOx = GPIOC;
                  SPI_Arch->PortMOSI.Pin   = 12;
              }
              else {                     // no pin remapping
                  SPI_Arch->PortNSS.GPIOx  = GPIOA;
                  SPI_Arch->PortNSS.Pin    = 15;
                  SPI_Arch->PortSCK.GPIOx  = GPIOB;
                  SPI_Arch->PortSCK.Pin    = 3;
                  SPI_Arch->PortMISO.GPIOx = GPIOB;
                  SPI_Arch->PortMISO.Pin   = 4;
                  SPI_Arch->PortMOSI.GPIOx = GPIOB;
                  SPI_Arch->PortMOSI.Pin   = 5;
              }
              break;
          }
          default: { return tue_DeviceNotFound; }
        }
    }
    if (1) { // compare chosen pin layout with configured pins
        //tue_InvalidPinConfig
        stm32_Port_t Port;
        switch (SPI_Index) { // check if board has defined the TX-pin of SPI to init
          
          case 1: { // check configuration of SPI #1
  #ifdef TTC_SPI1_MOSI
              stm32_loadPort(&Port, TTC_SPI1_MOSI);
              Assert(Port.GPIOx == SPI_Arch->PortMOSI.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == SPI_Arch->PortMOSI.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              SPI_Arch->PortMOSI.GPIOx = NULL;
  #endif
  #ifdef TTC_SPI1_MISO
              stm32_loadPort(&Port, TTC_SPI1_MISO);
              Assert(Port.GPIOx == SPI_Arch->PortMISO.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == SPI_Arch->PortMISO.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              SPI_Arch->PortMISO.GPIOx = NULL;
  #endif
  #ifdef TTC_SPI1_NSS
              stm32_loadPort(&Port, TTC_SPI1_NSS);
              Assert(Port.GPIOx == SPI_Arch->PortNSS.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == SPI_Arch->PortNSS.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              SPI_Arch->PortNSS.GPIOx = NULL;
  #endif
  #ifdef TTC_SPI1_SCK
              stm32_loadPort(&Port, TTC_SPI1_SCK);
              Assert(Port.GPIOx == SPI_Arch->PortSCK.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == SPI_Arch->PortSCK.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              SPI_Arch->PortSCK.GPIOx = NULL;
  #endif
              break;
          }
          case 2: { // check configuration of SPI #2
  #ifdef TTC_SPI2_MOSI
              stm32_loadPort(&Port, TTC_SPI2_MOSI);
              Assert(Port.GPIOx == SPI_Arch->PortMOSI.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == SPI_Arch->PortMOSI.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              SPI_Arch->PortMOSI.GPIOx = NULL;
  #endif
  #ifdef TTC_SPI2_MISO
              stm32_loadPort(&Port, TTC_SPI2_MISO);
              Assert(Port.GPIOx == SPI_Arch->PortMISO.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == SPI_Arch->PortMISO.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              SPI_Arch->PortMISO.GPIOx = NULL;
  #endif
  #ifdef TTC_SPI2_NSS
              stm32_loadPort(&Port, TTC_SPI2_NSS);
              Assert(Port.GPIOx == SPI_Arch->PortNSS.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == SPI_Arch->PortNSS.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              SPI_Arch->PortNSS.GPIOx = NULL;
  #endif
  #ifdef TTC_SPI2_SCK
              stm32_loadPort(&Port, TTC_SPI2_SCK);
              Assert(Port.GPIOx == SPI_Arch->PortSCK.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == SPI_Arch->PortSCK.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              SPI_Arch->PortSCK.GPIOx = NULL;
  #endif
              break;
          }
          case 3: { // check configuration of SPI #3
  #ifdef TTC_SPI3_MOSI
              stm32_loadPort(&Port, TTC_SPI3_MOSI);
              Assert(#.GPIOx == SPI_Arch->PortMOSI.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == SPI_Arch->PortMOSI.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              SPI_Arch->PortMOSI.GPIOx = NULL;
  #endif
  #ifdef TTC_SPI3_MISO
              stm32_loadPort(&Port, TTC_SPI3_MISO);
              Assert(Port.GPIOx == SPI_Arch->PortMISO.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == SPI_Arch->PortMISO.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              SPI_Arch->PortMISO.GPIOx = NULL;
  #endif
  #ifdef TTC_SPI3_NSS
              stm32_loadPort(&Port, TTC_SPI3_NSS);
              Assert(Port.GPIOx == SPI_Arch->PortNSS.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == SPI_Arch->PortNSS.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              SPI_Arch->PortNSS.GPIOx = NULL;
  #endif
  #ifdef TTC_SPI3_SCK
              stm32_loadPort(&Port, TTC_SPI3_SCK);
              Assert(Port.GPIOx == SPI_Arch->PortSCK.GPIOx, ec_InvalidArgument); // ERROR: configured GPIO cannot be used in current remapping layout!
              Assert(Port.Pin   == SPI_Arch->PortSCK.Pin,   ec_InvalidArgument); // ERROR: configured Pin  cannot be used in current remapping layout!
  #else
              SPI_Arch->PortSCK.GPIOx = NULL;
  #endif
              break;
          }
        default: Assert(0, ec_UNKNOWN); break; // No TTC_SPIn defined! Check your makefile.100_board_* file!
            
        }
    }
    SPI_I2S_DeInit(SPI_Arch->Base);
    if (1) { // activate clocks
        switch ( (u32_t) SPI_Arch->Base ) {
          case (u32_t) SPI1: {
              if (SPI_Generic->Layout) { // GPIOB + GPIOA
                  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1 | RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_AFIO, ENABLE);
              }
              else {                     // GPIOA
                  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1 | RCC_APB2Periph_GPIOA | RCC_APB2Periph_AFIO, ENABLE);
              }
              break;
          }
          case (u32_t) SPI2: {
              RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2 , ENABLE);
              RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB | RCC_APB2Periph_AFIO, ENABLE);
              break;
          }
          case (u32_t) SPI3: {
              if (SPI_Generic->Layout) { // GPPIOA, GPIOC
                  RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI3, ENABLE);
                  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOC | RCC_APB2Periph_AFIO, ENABLE);
              }
              else {                     // GPPIOA. GPPIOB
                  RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI3, ENABLE);
                  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_AFIO, ENABLE);
              }
              break;
          }
          default: Assert(0, ec_UNKNOWN); break; // should never occur!
        }
    }
    if (1) { // configure GPIOs

        if (SPI_Generic->Flags.Bits.Master) {
            if (SPI_Arch->PortMOSI.GPIOx)  stm32_initPort2(&(SPI_Arch->PortMOSI), GPIO_Mode_AF_PP,       GPIO_Speed_50MHz);
            if (SPI_Arch->PortMISO.GPIOx)  stm32_initPort2(&(SPI_Arch->PortMISO), GPIO_Mode_IN_FLOATING, GPIO_Speed_50MHz);
            if (SPI_Arch->PortSCK.GPIOx)   stm32_initPort2(&(SPI_Arch->PortSCK),  GPIO_Mode_AF_PP,       GPIO_Speed_50MHz);
            if (SPI_Arch->PortNSS.GPIOx)   stm32_initPort2(&(SPI_Arch->PortNSS),  GPIO_Mode_AF_PP,       GPIO_Speed_50MHz);
        }
        else {
            if (SPI_Arch->PortMOSI.GPIOx)  stm32_initPort2(&(SPI_Arch->PortMOSI), GPIO_Mode_IN_FLOATING, GPIO_Speed_50MHz);
            if (SPI_Arch->PortMISO.GPIOx)  stm32_initPort2(&(SPI_Arch->PortMISO), GPIO_Mode_AF_PP,       GPIO_Speed_50MHz);
            if (SPI_Arch->PortSCK.GPIOx)   stm32_initPort2(&(SPI_Arch->PortSCK),  GPIO_Mode_IN_FLOATING, GPIO_Speed_50MHz);
            if (SPI_Arch->PortNSS.GPIOx)   stm32_initPort2(&(SPI_Arch->PortNSS),  GPIO_Mode_IN_FLOATING, GPIO_Speed_50MHz);
        }
    }
    if (1) { // apply remap layout

        switch ( (u32_t) SPI_Arch->Base ) {
          case (u32_t) SPI1: { // SPI1
              if (SPI_Generic->Layout)
                  GPIO_PinRemapConfig(GPIO_Remap_SPI1, ENABLE);
              else
                  GPIO_PinRemapConfig(GPIO_Remap_SPI1, DISABLE);
  
              break;
          }
          case (u32_t) SPI3: { // SPI3
              if (SPI_Generic->Layout)   // (-> ST RM0041 STM32F100xx advanced ARM-based 32-bit MCUs p.118)
                  GPIO_PinRemapConfig(GPIO_Remap_SPI3, ENABLE);
              else                      // no pin remapping
                  GPIO_PinRemapConfig(GPIO_Remap_SPI3, DISABLE);
              break;
          }
  
          default: { break; }
        }
    }
    if (1) { // init SPI

        SPI_InitTypeDef SPI_MyInit;
        memset( &SPI_MyInit, 0, sizeof(SPI_InitTypeDef) );

        if (SPI_Generic->Flags.Bits.Transmit && SPI_Generic->Flags.Bits.Receive) {
          if (SPI_Generic->Flags.Bits.Bidirectional)   SPI_MyInit.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
          else                                         SPI_MyInit.SPI_Direction = SPI_Direction_1Line_Rx | SPI_Direction_1Line_Tx;
        }
        else if (SPI_Generic->Flags.Bits.Receive)      SPI_MyInit.SPI_Direction = SPI_Direction_2Lines_RxOnly;
        
        if (SPI_Generic->Flags.Bits.Master)            SPI_MyInit.SPI_Mode = SPI_Mode_Master;
        else                                           SPI_MyInit.SPI_Mode = SPI_Mode_Slave;
        
        if (SPI_Generic->Flags.Bits.WordSize16)        SPI_MyInit.SPI_DataSize = SPI_DataSize_16b;
        else                                           SPI_MyInit.SPI_DataSize = SPI_DataSize_8b;
        
        if (SPI_Generic->Flags.Bits.ClockIdleHigh)     SPI_MyInit.SPI_CPOL = SPI_CPOL_High;
        else                                           SPI_MyInit.SPI_CPOL = SPI_CPOL_Low;
        
        if (SPI_Generic->Flags.Bits.ClockPhase2ndEdge) SPI_MyInit.SPI_CPHA = SPI_CPHA_2Edge;
        else                                           SPI_MyInit.SPI_CPHA = SPI_CPHA_1Edge;
        
        if (SPI_Generic->Flags.Bits.HardwareNSS)       SPI_MyInit.SPI_NSS = SPI_NSS_Hard;
        else                                           SPI_MyInit.SPI_NSS = SPI_NSS_Soft;
        
        if (SPI_Generic->Flags.Bits.FirstBitMSB)       SPI_MyInit.SPI_FirstBit = SPI_FirstBit_MSB;
        else                                           SPI_MyInit.SPI_FirstBit = SPI_FirstBit_LSB;

        SPI_MyInit.SPI_BaudRatePrescaler = SPI_Generic->BaudRatePrescaler;
        if (SPI_Generic->Flags.Bits.CRC8 || SPI_Generic->Flags.Bits.CRC16) {
            SPI_MyInit.SPI_CRCPolynomial = SPI_Generic->CRC_Polynom;
            SPI_CalculateCRC(SPI_Arch->Base, ENABLE);
        }
        else {
            SPI_MyInit.SPI_CRCPolynomial = 0;
            SPI_CalculateCRC(SPI_Arch->Base, DISABLE);
        }
        
        
        // translate SPI_MyInit back to SPI_Generic
        if (SPI_MyInit.SPI_Direction & SPI_Direction_2Lines_FullDuplex) {
          SPI_Generic->Flags.Bits.Bidirectional = 1;
          SPI_Generic->Flags.Bits.Simplex       = 0;
          SPI_Generic->Flags.Bits.Transmit      = 1;
          SPI_Generic->Flags.Bits.Receive       = 1;
        }
        else if (SPI_MyInit.SPI_Direction & SPI_Direction_2Lines_RxOnly) {
          SPI_Generic->Flags.Bits.Bidirectional = 0;
          SPI_Generic->Flags.Bits.Simplex       = 0;
          SPI_Generic->Flags.Bits.Transmit      = 0;
          SPI_Generic->Flags.Bits.Receive       = 1;
          
        }
        else {
          SPI_Generic->Flags.Bits.Bidirectional = 0;
          SPI_Generic->Flags.Bits.Simplex       = 1;
          
          SPI_Generic->Flags.Bits.Receive       = 1;
          if (SPI_MyInit.SPI_Direction & SPI_Direction_1Line_Tx)  SPI_Generic->Flags.Bits.Transmit = 1;
          if (SPI_MyInit.SPI_Direction & SPI_Direction_1Line_Rx)  SPI_Generic->Flags.Bits.Receive  = 1;
          
        }
        
        if (SPI_MyInit.SPI_Mode == SPI_Mode_Master)       SPI_Generic->Flags.Bits.Master = 1;
        else                                              SPI_Generic->Flags.Bits.Master = 0;
        if (SPI_MyInit.SPI_DataSize == SPI_DataSize_16b)  SPI_Generic->Flags.Bits.WordSize16 = 1;
        else                                              SPI_Generic->Flags.Bits.WordSize16 = 0;
        
        if (SPI_MyInit.SPI_CPOL == SPI_CPOL_High) SPI_Generic->Flags.Bits.ClockIdleHigh = 1;
        else                                      SPI_Generic->Flags.Bits.ClockIdleHigh = 0;
        if (SPI_MyInit.SPI_CPHA == SPI_CPHA_2Edge) SPI_Generic->Flags.Bits.ClockPhase2ndEdge = 1;
        else                                       SPI_Generic->Flags.Bits.ClockPhase2ndEdge = 0;
        if (SPI_MyInit.SPI_NSS == SPI_NSS_Soft) SPI_Generic->Flags.Bits.HardwareNSS = 0;
        else                                    SPI_Generic->Flags.Bits.HardwareNSS = 1;
        if (SPI_MyInit.SPI_FirstBit == SPI_FirstBit_LSB) SPI_Generic->Flags.Bits.FirstBitMSB = 1;
        else                                             SPI_Generic->Flags.Bits.FirstBitMSB = 0;

        SPI_Generic->BaudRatePrescaler = SPI_MyInit.SPI_BaudRatePrescaler;
        SPI_Generic->CRC_Polynom       = SPI_MyInit.SPI_CRCPolynomial;

        SPI_Init(SPI_Arch->Base, &SPI_MyInit);

        if (SPI_Generic->Flags.Bits.Master) { // workaround: SSOE is not configured by StdPeripheralLib!
            if (SPI_Generic->Flags.Bits.HardwareNSS)
                ( (SPI_t*) SPI_Arch->Base )->CR2.SSOE = 1;
            else
                ( (SPI_t*) SPI_Arch->Base )->CR2.SSOE = 0;
        }
    }
    if (0) { // setup interrupts
        if (SPI_Generic->Flags.Bits.Irq_Error)
          SPI_I2S_ITConfig(SPI_Arch->Base, SPI_I2S_IT_ERR, ENABLE);
        
        if (SPI_Generic->Flags.Bits.Irq_Received)
          SPI_I2S_ITConfig(SPI_Arch->Base, SPI_I2S_IT_RXNE, ENABLE);
        
        if (SPI_Generic->Flags.Bits.Irq_Transmitted)
          SPI_I2S_ITConfig(SPI_Arch->Base, SPI_I2S_IT_TXE, ENABLE);

        /* Enable and set SPIn Interrupt to the lowest priority */
        NVIC_InitTypeDef NVIC_InitStructure;
        switch ( (u32_t) SPI_Arch->Base ) {

#ifdef SPI1_IRQn
          case (u32_t) SPI1: NVIC_InitStructure.NVIC_IRQChannel=SPI1_IRQn; break;
#endif
#ifdef SPI2_IRQn
          case (u32_t) SPI2: NVIC_InitStructure.NVIC_IRQChannel=SPI2_IRQn; break;
#endif
#ifdef SPI3_IRQn
          case (u32_t) SPI3: NVIC_InitStructure.NVIC_IRQChannel=SPI3_IRQn; break;
#endif

          default: { break; }
        }

        NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority=0x0F;
        NVIC_InitStructure.NVIC_IRQChannelSubPriority=0x0F;
        NVIC_InitStructure.NVIC_IRQChannelCmd=ENABLE;
        NVIC_Init(&NVIC_InitStructure);
    }
    
    SPI_Cmd(SPI_Arch->Base, ENABLE);
    stm32_SPIs[SPI_Index-1]=(SPI_t*) SPI_Arch->Base;
    
    return tue_OK;
}
ttc_spi_errorcode_e stm32_spi_sendBytes(u8_t SPI_Index, const char* Buffer, u16_t Amount) {
  SPI_t* MySPI = stm32_SPIs[SPI_Index-1];
  Assert(MySPI != NULL, ec_NULL);

  while (Amount-- > 0) {
    _stm32_spi_sendWord(MySPI, (u16_t) *Buffer++);
  }
  return tue_OK;
}
ttc_spi_errorcode_e stm32_spi_sendASCII(u8_t SPI_Index, const char* Buffer, u16_t MaxLength) {
    SPI_t* MySPI = stm32_SPIs[SPI_Index-1];
    Assert(MySPI != NULL, ec_NULL);

    char C=0;
    while ( (MaxLength-- > 0) && ((C=*Buffer++) != 0)  ) {
      _stm32_spi_sendWord(MySPI, (u16_t) C);
    }
    return tue_OK;
}
ttc_spi_errorcode_e stm32_spi_sendWord(u8_t SPI_Index, const u16_t Word) {
    SPI_t* MySPI = stm32_SPIs[SPI_Index-1];
    Assert(MySPI != NULL, ec_NULL);

    _stm32_spi_sendWord(MySPI, Word);
    return tue_OK;
}
ttc_spi_errorcode_e stm32_spi_readByte(u8_t SPI_Index, char* Byte, u32_t TimeOut) {
    SPI_t* MySPI = stm32_SPIs[SPI_Index-1];
    Assert(MySPI != NULL, ec_NULL);

    if ( _stm32_spi_waitForRXNE(MySPI, TimeOut) ) return tue_TimeOut;

    *Byte=(char) 0xff & SPI_I2S_ReceiveData( (SPI_TypeDef*) MySPI );
    // MySPI->SPI_SR.RXNE=0;

    return tue_OK;
}
ttc_spi_errorcode_e stm32_spi_readWord(u8_t SPI_Index, u16_t* Word, u32_t TimeOut) {
    SPI_t* MySPI = stm32_SPIs[SPI_Index-1];
    Assert(MySPI != NULL, ec_NULL);

    if ( _stm32_spi_waitForRXNE(MySPI, TimeOut) ) return tue_TimeOut;

    *Word=SPI_I2S_ReceiveData( (SPI_TypeDef*) MySPI  );
    // MySPI->SPI_SR.RXNE=0;

    return tue_OK;
}

//} Function definitions
//{ private functions (ideally) -------------------------------------------------

void _stm32_spi_sendWord(SPI_t* MySPI, const u16_t Word) {
    Assert(MySPI != NULL, ec_NULL);

    while (SPI_I2S_GetFlagStatus( (SPI_TypeDef*) MySPI , SPI_I2S_FLAG_TXE) == RESET) uSleep(10);
    SPI_I2S_SendData( (SPI_TypeDef*) MySPI , Word);
}
ttc_spi_errorcode_e _stm32_spi_waitForRXNE(SPI_t* MySPI, u32_t TimeOut) {
    if (TimeOut > 0) { // wait until word received or timeout
        while ( (TimeOut > 0) && (MySPI->SR.RXNE == 0) ) {
            if (TimeOut > 9) TimeOut -= 10;
            else             TimeOut=0;
            uSleep(10);
        }
        if (TimeOut == 0)  return tue_TimeOut;
    }
    else {             // wait until word received (no timeout)
        while (MySPI->SR.RXNE == 0)
            uSleep(10);
    }

    return tue_OK;
}

//} private functions
