/*{ Template_FreeRTOS::.c ************************************************

     Several different boxes run over the screen.
     Each box is controlled by one task.
     
     written 2012 by Ioanna Tsekoura, Gregor Rebel
     
}*/

#include "example_lcd_boxes.h"

u16_t LCD_Width  = 240;
u16_t LCD_Height = 320;
u16_t Color_Background = LCD_COLOR_YELLOW;

//{ Function definitions *************************************************

void lcd_boxes_initPorts() {

    //{ init_LCD Start
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);

    GPIO_InitTypeDef GPIO_InitStructure;
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_Init(GPIOC, &GPIO_InitStructure);
    GPIO_Init(GPIOD, &GPIO_InitStructure);
    GPIO_Init(GPIOE, &GPIO_InitStructure);
    GPIO_Init(GPIOF, &GPIO_InitStructure);
    GPIO_Init(GPIOG, &GPIO_InitStructure);

    /* Enable the FSMC Clock */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE);

    LCD_Init();

    //} init_LCD Ende

    LCD_SetBacklight(1);
    LCD_Clear(Color_Background);

}
void lcd_start_boxes() {

    xQueueHandle DrawQueue = xQueueCreate(10, sizeof(DrawData_t));
    Assert(NULL != DrawQueue, ec_Malloc);

    lcd_boxes_initPorts();

    if (1) {
        DrawData_t* Box = newBox(DrawQueue, LCD_COLOR_BLUE, 1, 0, 15, 15, 4, 5);
        Assert(0 != Box, ec_Malloc);
    }
    if (1) {
        DrawData_t* Box = newBox(DrawQueue, LCD_COLOR_RED, 50, 100, 6, 9, -4, 6);
        Assert(0 != Box, ec_Malloc);
    }
    if (1) {
        DrawData_t* Box = newBox(DrawQueue, LCD_COLOR_GREEN, 100, 200, 15, 10, 7, -6);
        Assert(0 != Box, ec_Malloc);
    }
    if (1) {
        DrawData_t* Box = newBox(DrawQueue, LCD_COLOR_GRAY, 0, 300, 6, 6, -2, -3);
        Assert(0 != Box, ec_Malloc);
    }

    portBASE_TYPE Task2 = xTaskCreate(tDrawIt,                          // function to start as thread
                                      (const signed char*) "DrawsRect",        // thread name (just for debugging)
                                      (size_t) 1 * configMINIMAL_STACK_SIZE,                      // stack size
                                      (void *) DrawQueue,              // passed as argument to taskLEDs()
                                      (unsigned portBASE_TYPE) 1,        // task priority (higher values mean more process time)
                                      (xTaskHandle*) NULL                // can return a handle to created task
                                      );
    Assert(0 != Task2, ec_Malloc);
}
DrawData_t* newBox(xQueueHandle DrawingQueue, u16_t Color, s16_t X, s16_t Y, u8_t Width, u8_t Height, s8_t SpeedX, s8_t SpeedY) {
    drawTaskArgs_t* NewBox = (drawTaskArgs_t*) pvPortMalloc( sizeof(drawTaskArgs_t) );

    if (NewBox != NULL) {
        NewBox->DrawQueue = DrawingQueue;
        NewBox->Box2Draw.Type = ddt_MovingBox;
        NewBox->Box2Draw.Polygon.MovingBox.Color  = Color;
        NewBox->Box2Draw.Polygon.MovingBox.PosX   = X;
        NewBox->Box2Draw.Polygon.MovingBox.PosY   = Y;
        NewBox->Box2Draw.Polygon.MovingBox.Width  = Width;
        NewBox->Box2Draw.Polygon.MovingBox.Height = Height;
        NewBox->Box2Draw.Polygon.MovingBox.SpeedX = SpeedX;
        NewBox->Box2Draw.Polygon.MovingBox.SpeedY = SpeedY;

        portBASE_TYPE Task = xTaskCreate(tMoveBox,                             // function to start as thread
                                         (const signed char*) NULL,             // thread name (just for debugging)
                                         (size_t) 2 * configMINIMAL_STACK_SIZE, // stack size
                                         (void *) NewBox,                       // passed as argument to task-function
                                         (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                                         (xTaskHandle*) NULL                    // can return a handle to created task
                                         );
        Assert(Task != 0, ec_Malloc); 
    }
    return &(NewBox->Box2Draw);
}
void tMoveBox(void *pvParameters) {

    drawTaskArgs_t* Arguments = (drawTaskArgs_t*) pvParameters;
    xQueueHandle DrawQueue = Arguments->DrawQueue;

    Assert(NULL != DrawQueue, ec_InvalidArgument);

    DrawData_t* DrawMovingBox = &(Arguments->Box2Draw);
    elb_MovingBox_t* MovingBox = &(DrawMovingBox->Polygon.MovingBox);

    DrawData_t DrawClearBox;
    DrawClearBox.Type = ddt_MovingBox;
    elb_MovingBox_t* ClearBox = &(DrawClearBox.Polygon.MovingBox);
    ClearBox->Color  = Color_Background;
    ClearBox->SpeedX = 0;
    ClearBox->SpeedY = 0;

    DrawData_t DrawClearLPolygon;
    DrawClearLPolygon.Type = ddt_L_Polygon;
    elb_L_Polygon_t* ClearLPolygon = &(DrawClearLPolygon.Polygon.L_Polygon);
    ClearLPolygon->Color  = Color_Background;

    while (1) {

        int PosX = MovingBox->PosX;
        int PosY = MovingBox->PosY;
        int NewPosX = PosX + MovingBox->SpeedX;
        int NewPosY = PosY + MovingBox->SpeedY;

        int Border_Right = LCD_Width  - MovingBox->Width;
        int Border_Lower = LCD_Height - MovingBox->Height;

        if (NewPosX > Border_Right) {
            MovingBox->SpeedX = -MovingBox->SpeedX;
            int Excess = NewPosX - Border_Right;
            NewPosX -= Excess;
        }
        else if (NewPosX < 0) {
            MovingBox->SpeedX = -MovingBox->SpeedX;
            NewPosX = -NewPosX;
        }
        if (NewPosY > Border_Lower) {
            MovingBox->SpeedY = -MovingBox->SpeedY;
            int Excess = NewPosY - Border_Lower;
            NewPosY -= Excess;
        }
        else if (NewPosY < 0) {
            MovingBox->SpeedY = -MovingBox->SpeedY;
            NewPosY = -NewPosY;
        }

        if (1) { // 1st implementation, clear the previous box and redraw all of it
            ClearBox->PosX = PosX;
            ClearBox->PosY = PosY;
            ClearBox->Width  = MovingBox->Width;
            ClearBox->Height = MovingBox->Height;
            queueInsert(DrawQueue, &DrawClearBox);

            MovingBox->PosX = NewPosX;
            MovingBox->PosY = NewPosY;
            queueInsert(DrawQueue, DrawMovingBox);
        }
        else {   // 2nd implementation: clear as few pixels as possible

            if ((MovingBox->PosX==NewPosX)&&(MovingBox->PosY==NewPosY)) {
                // clear nothing
                MovingBox->PosX=NewPosX;
                MovingBox->PosY=NewPosY;
                queueInsert(DrawQueue, DrawMovingBox);
            }
            else if ((MovingBox->PosX==NewPosX)&&(MovingBox->PosY<NewPosY)) {
                if ((MovingBox->PosY+MovingBox->Height)<NewPosY) {
                    ClearLPolygon->ExternalX = PosX;
                    ClearLPolygon->ExternalY = PosY;
                    ClearLPolygon->Width     = MovingBox->Width;
                    ClearLPolygon->Height    = MovingBox->Height;
                    ClearLPolygon->InternalX = 0;
                    ClearLPolygon->InternalY = 0;

                    //X box_param DrawClearBox={bg_color,MovingBox->PosX,MovingBox->PosY,MovingBox->Width,MovingBox->Height,0,0};
                    queueInsert(DrawQueue, &DrawClearLPolygon);

                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
                else {
                    //box_param DrawClearBox={bg_color,MovingBox->PosX,MovingBox->PosY,MovingBox->Width,(NewPosY-MovingBox->PosY),0,0};
                    ClearLPolygon->ExternalX = PosX;
                    ClearLPolygon->ExternalY = PosY;
                    ClearLPolygon->Width     = MovingBox->Width;
                    ClearLPolygon->Height    = NewPosY - MovingBox->PosY;
                    ClearLPolygon->InternalX = 0;
                    ClearLPolygon->InternalY = 0;

                    queueInsert(DrawQueue, &DrawClearLPolygon);
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
            }
            else if ((MovingBox->PosX==NewPosX)&&(MovingBox->PosY>NewPosY)) {
                if ((NewPosY+MovingBox->Height)<MovingBox->PosY) {
                    //box_param DrawClearBox={bg_color,MovingBox->PosX,MovingBox->PosY,MovingBox->Width,MovingBox->Height,0,0};
                    ClearLPolygon->ExternalX = PosX;
                    ClearLPolygon->ExternalY = PosY;
                    ClearLPolygon->Width     = MovingBox->Width;
                    ClearLPolygon->Height    = MovingBox->Height;
                    ClearLPolygon->InternalX = 0;
                    ClearLPolygon->InternalY = 0;

                    queueInsert(DrawQueue, &DrawClearLPolygon);
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                }
                else {
                    /*
                    box_param DrawClearBox={bg_color,MovingBox->PosX,(NewPosY+MovingBox->Height),MovingBox->Width,(MovingBox->PosY-NewPosY),0,0};
                    queueInsert(DrawQueue, &DrawClearBox);
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                    */
                }
            }
            else if ((MovingBox->PosX<NewPosX)&&(MovingBox->PosY==NewPosY)) {
                if ((MovingBox->PosX+MovingBox->Width)<NewPosX) {
                    /*
                    box_param DrawClearBox={bg_color,MovingBox->PosX,MovingBox->PosY,MovingBox->Width,MovingBox->Height,0,0};
                    queueInsert(DrawQueue, &DrawClearBox);
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                    */
                }
                else {
                    /*
                    box_param DrawClearBox={bg_color,MovingBox->PosX,MovingBox->PosY,(NewPosX-MovingBox->PosX),MovingBox->Height,0,0};
                    queueInsert(DrawQueue, &DrawClearBox);
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                    */
                }
            }
            else if ((MovingBox->PosX>NewPosX)&&(MovingBox->PosY==NewPosY)) {
                if ((NewPosX+MovingBox->Width)<MovingBox->PosX) {
                    /*
                    box_param DrawClearBox={bg_color,MovingBox->PosX,MovingBox->PosY,MovingBox->Width,MovingBox->Height,0,0};
                    queueInsert(DrawQueue, &DrawClearBox);
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                    */
                }
                else {
                    /*
                    box_param DrawClearBox={bg_color,NewPosX+MovingBox->Width,MovingBox->PosY,(MovingBox->PosX-NewPosX),MovingBox->Height,0,0};
                    queueInsert(DrawQueue, &DrawClearBox);
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                    */
                }
            }

            else if ((MovingBox->PosX<NewPosX)&&(MovingBox->PosY<NewPosY)) { /*Left to Right. Top to Bottom.*/
                if ((NewPosX>MovingBox->PosX+MovingBox->Width)||(NewPosY>MovingBox->PosY+MovingBox->Height)) {
                    /*
                    box_param DrawClearBox={bg_color,MovingBoxv.PosX,MovingBox->PosY,MovingBox->Width,MovingBox->Height,0,0};
                    queueInsert(DrawQueue, &DrawClearBox);
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                    */
                }
                else {
                    /*
                    box_param DrawClearBox={bg_color,MovingBox->PosX,MovingBox->PosY,MovingBox->Width,MovingBox->Height,NewPosX,NewPosY};
                    queueInsert(DrawQueue, &DrawClearBox);
                    box_param Sth={MovingBox->BoxColor,(NewPosX+MovingBox->Width),(NewPosY+MovingBox->Height),MovingBox->Width,MovingBox->Height,(MovingBox->PosX+MovingBox->Width),(MovingBox->PosY+MovingBox->Height)};
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, &Sth);
                    */
                }
            }
            else if ((MovingBox->PosX<NewPosX)&&(MovingBox->PosY>NewPosY)) { /*Left to Right. Bottom to Top*/
                if ((NewPosX>MovingBox->PosX+MovingBox->Width)||((NewPosY+MovingBox->Height)<MovingBox->PosY)) {
                    /*
                    box_param DrawClearBox={bg_color,MovingBox->PosX,MovingBox->PosY,MovingBox->Width,MovingBox->Height,0,0};
                    queueInsert(DrawQueue, &DrawClearBox);
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                    */
                }
                else {
                    /*
                    box_param DrawClearBox={bg_color,MovingBox->PosX,(MovingBox->PosY+MovingBox->Height),MovingBox->Width,MovingBox->Height,NewPosX,(NewPosY+MovingBox->Height)};
                    queueInsert(DrawQueue, &DrawClearBox);
                    box_param Sth={MovingBox->BoxColor,(NewPosX+MovingBox->Width),NewPosY,MovingBox->Width,MovingBox->Height,(MovingBox->PosX+MovingBox->Width),MovingBox->PosY};
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, &Sth);
                    */
                }
            }
            else if ((MovingBox->PosX>NewPosX)&&(MovingBox->PosY<NewPosY)) { /*Right to Left. Top to Bottom*/
                if (((NewPosX+MovingBox->Width)<MovingBox->PosX)||(NewPosY>MovingBox->PosY+MovingBox->Height)) {
                    /*
                    box_param DrawClearBox={bg_color,MovingBox->PosX,MovingBox->PosY,MovingBox->Width,MovingBox->Height,0,0};
                    queueInsert(DrawQueue, &DrawClearBox);
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                    */
                }
                else {
                    /*
                    box_param DrawClearBox={bg_color,(MovingBox->PosX+MovingBox->Width),MovingBox->PosY,MovingBox->Width,MovingBox->Height,(NewPosX+MovingBox->Width),NewPosY};
                    queueInsert(DrawQueue, &DrawClearBox);
                    box_param Sth={MovingBox->BoxColor,NewPosX,(NewPosY+MovingBox->Height),MovingBox->Width,MovingBox->Height,MovingBox->PosX,(MovingBox->PosY+MovingBox->Height)};
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, &Sth);
                    */
                }
            }
            else if ((MovingBox->PosX>NewPosX)&&(MovingBox->PosY>NewPosY)) {
                if (((NewPosX+MovingBox->Width)<MovingBox->PosX)||((NewPosY+MovingBox->Height)<MovingBox->PosY)) {
                    /*
                    box_param DrawClearBox={bg_color,MovingBox->PosX,MovingBox->PosY,MovingBox->Width,MovingBox->Height,0,0};
                    queueInsert(DrawQueue, &DrawClearBox);
                    MovingBox->PosX=NewPosX;
                    MovingBox->PosY=NewPosY;
                    queueInsert(DrawQueue, DrawMovingBox);
                    */
                }
                else {
                   /*
                   box_param DrawClearBox={bg_color,(MovingBox->PosX+MovingBox->Width),(MovingBox->PosY+MovingBox->Height),MovingBox->Width,MovingBox->Height,(NewPosX+MovingBox->Width),(NewPosY+MovingBox->Height)};
                   queueInsert(DrawQueue, &DrawClearBox);
                   box_param Sth={MovingBox->BoxColor,NewPosX,NewPosY,MovingBox->Width,MovingBox->Height,MovingBox->PosX,MovingBox->PosY};
                   MovingBox->PosX=NewPosX;
                   MovingBox->PosY=NewPosY;
                   queueInsert(DrawQueue, &Sth);
                   */
                }
            }

        }

        mSleep(30);
    }
}
void queueInsert(xQueueHandle DrawQueue, DrawData_t *Box) {

    while (xQueueSend(DrawQueue, Box, (portTickType) -1) != pdTRUE) 
      taskYIELD();

}
void tDrawIt(void *pvParameters) {
    xQueueHandle DrawQueue= (xQueueHandle) pvParameters;
    Assert(NULL != DrawQueue, ec_InvalidArgument);

    while (1) {
        DrawData_t Box2Draw;
        while ( xQueueReceive(DrawQueue, &Box2Draw, (portTickType) -1) ) {
            if (Box2Draw.Type == ddt_MovingBox) {
                LCD_DrawRect(Box2Draw.Polygon.MovingBox.PosX,
                             Box2Draw.Polygon.MovingBox.PosY,
                             Box2Draw.Polygon.MovingBox.Width,
                             Box2Draw.Polygon.MovingBox.Height,
                             Box2Draw.Polygon.MovingBox.Color
                             );
            }
            else {
                LCD_DrawLpolygon(Box2Draw.Polygon.L_Polygon.ExternalX,
                                 Box2Draw.Polygon.L_Polygon.ExternalY,
                                 Box2Draw.Polygon.L_Polygon.Width,
                                 Box2Draw.Polygon.L_Polygon.Height,
                                 Box2Draw.Polygon.L_Polygon.InternalX,
                                 Box2Draw.Polygon.L_Polygon.InternalY,
                                 Box2Draw.Polygon.L_Polygon.Color
                                );
            }
        }
        taskYIELD();
    }
}
void LCD_DrawLpolygon(int ExternalX, int ExternalY, int Width, int Height, int InternalX, int InternalY, u16 Color)
{
    int i;

    if ((ExternalX<InternalX)&&(ExternalY<InternalY)) {
        int k=ExternalY;
        for(i=0; i<Height; i++, k++) {
            u32 addr = k*0x100 + ExternalX;
            LCD_FillMem(addr, Color, (InternalX-ExternalX));
        }
        k=ExternalY;
        for(i=0; i<(InternalY-ExternalY); i++, k++) {
            u32 addr = k*0x100 + InternalX;
            LCD_FillMem(addr, Color, (ExternalX+Width-InternalX));
        }
    }
    else if ((ExternalX<InternalX)&&(ExternalY>InternalY)) {
        int k=ExternalY-Height;
        for(i=0; i<Height; i++, k++) {
            u32 addr = k*0x100 + ExternalX;
            LCD_FillMem(addr, Color, (InternalX-ExternalX));
        }
        k=InternalY;
        for(i=0; i<(ExternalY-InternalY); i++, k++) {
            u32 addr = k*0x100 + InternalX;
            LCD_FillMem(addr, Color, (ExternalX+Width-InternalX));
        }
    }
    else if ((ExternalX>InternalX)&&(ExternalY<InternalY)) {
        int k=ExternalY;
        for(i=0; i<(InternalY-ExternalY); i++, k++) {
            u32 addr = k*0x100 + ExternalX-Width;
            LCD_FillMem(addr, Color, Width);
        }
        k=InternalY;
        for(i=0; i<(ExternalY+Height-InternalY); i++, k++) {
            u32 addr = k*0x100 + InternalX;
            LCD_FillMem(addr, Color, (ExternalX-InternalX));
        }
    }
    else {
        int k=InternalY;
        for(i=0; i<(ExternalY-InternalY); i++, k++) {
            u32 addr = k*0x100 + ExternalX-Width;
            LCD_FillMem(addr, Color, Width);
        }
        k=ExternalY-Height;
        for(i=0; i<(InternalY+Height-ExternalY); i++, k++) {
            u32 addr = k*0x100 + InternalX;
            LCD_FillMem(addr, Color, (ExternalX-InternalX));
        }
    }
}
//} Function definitions
