/*{ TemplateName::.c ************************************************
 
 Empty template for new c-files.
 Copy and adapt to your needs.
 
}*/

#include "example_spi.h"

//{ Global variables *************************************************

#ifdef TTC_LED1
    ttc_Port_t Port_LED1;
#endif

ttc_spi_generic_t       SPI_Generic;  // architecture independent SPI configuration
ttc_spi_architecture_u  SPI_Arch;     // architecture specific SPI configuration
#define SPI_INDEX 1                   // using first SPI available on current board

//} Global variables
//{ Function definitions *************************************************

void example_spi_init() {
#ifdef TTC_LED1
    ttc_initPort(&Port_LED1, TTC_LED1, tgm_output_push_pull);
    ttc_portSet(&Port_LED1);
#endif

    u8_t MaxSPI = ttc_getMax_SPI_Index();
    Assert(MaxSPI > 0, ec_UNKNOWN);

    ttc_spi_errorcode_e Error;

    Error = ttc_getSPI_Defaults(SPI_INDEX, &SPI_Generic);
    Assert( Error == tue_OK, ec_UNKNOWN);

    #ifdef EXTENSION_600_example_spi_master
    SPI_Generic.Flags.Bits.Master = 1;
    #else
    SPI_Generic.Flags.Bits.Master = 0;
    #endif
    SPI_Generic.Flags.Bits.CRC8 = 0;

    //SPI_Generic.Flags.Bits.
    Error = ttc_initSPI(SPI_INDEX, &SPI_Generic, &SPI_Arch);
    Assert( Error == tue_OK, ec_UNKNOWN);
}
void example_spi_start() {
    example_spi_init();

#ifdef EXTENSION_600_example_spi_master
    xTaskCreate(task_SPI_sender,                   // function to start as thread
                (const signed char*) "SPItx",      // thread name (just for debugging)
                (size_t) 512,                      // stack size
                (void *) NULL,                     // passed as argument to taskLEDs()
                (unsigned portBASE_TYPE) 1,        // task priority (higher values mean more process time)
                (xTaskHandle*) NULL                // can return a handle to created task
                );
#else
    xTaskCreate(task_SPI_listener,                 // function to start as thread
                (const signed char*) "SPIrx",      // thread name (just for debugging)
                (size_t) 512,                      // stack size
                (void *) NULL,                     // passed as argument to taskLEDs()
                (unsigned portBASE_TYPE) 1,        // task priority (higher values mean more process time)
                (xTaskHandle*) NULL                // can return a handle to created task
                );
#endif
}
void task_SPI_sender(void *pvParameters) {
    pvParameters = pvParameters; // avoids warning: unused parameter 'pvParameters'

#ifdef TTC_LED1
    ttc_portClr(&Port_LED1);
#endif

    while (1) {
#ifdef TTC_LED1
        ttc_portSet(&Port_LED1);
#endif
        sendData("Hello world!\n\r", -1);
#ifdef TTC_LED1
        ttc_portClr(&Port_LED1);
#endif
        mSleep(300);
    }
}
void task_SPI_listener(void *pvParameters) {
    pvParameters = pvParameters; // avoids warning: unused parameter 'pvParameters'

#define Size_ReceiveBuffer 20
    char ReceiveBuffer[Size_ReceiveBuffer];
    ReceiveBuffer[0] = 0;
    char* Writer = ReceiveBuffer;
    char* WriterEnd = Writer + Size_ReceiveBuffer - 1;

#ifdef TTC_LED1
    ttc_portClr(&Port_LED1);
#endif

    while (1) {
        ttc_spi_errorcode_e Status = ttc_readByte(SPI_INDEX, Writer, 30000);

        while (Status == tue_OK) { // store all incoming bytes
#ifdef TTC_LED1
          ttc_portSet(&Port_LED1);
#endif
          char Byte = *Writer;
          Writer++;

          if ( (Byte == 13) ||       // carriage return found
               (Writer == WriterEnd) // buffer full
               )
          {
              *Writer = 0;            // terminate string
              if (0) processData(ReceiveBuffer, Size_ReceiveBuffer);
              Writer = ReceiveBuffer; // back to start of buffer
          }

          // check for next byte
          Status = ttc_readByte(SPI_INDEX, Writer, 30);
#ifdef TTC_LED1
          ttc_portClr(&Port_LED1);
#endif
        }
        uSleep(1);
    }
}
void processData(char* Data, u8_t MaxLength) {
    ttc_sendASCII(SPI_INDEX, "You said '\0", -1);
    ttc_sendASCII(SPI_INDEX, Data, MaxLength);
    ttc_sendASCII(SPI_INDEX, "'\n\0", -1);
}
void sendData(const char* Buffer, u16_t MaxLength) {
    Assert(Buffer != 0, ec_InvalidArgument);

    char C = 0;
    while ( (MaxLength-- > 0) && ((C=*Buffer++) != 0)  ) {
      ttc_sendWord(SPI_INDEX, (u16_t) C);
      mSleep(5);
    }
}

//} Function definitions
